/*  tds200mt
 *  Copyright (C) 1999 Alvaro De Amicis
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "tmt.h"
#include "tmtif.h"
#include "tmtsig.h"
#include "utils.h"
#include "graphics.h"
#include "comm.h"

#include <getopt.h>

/* globals */
CommParam comm_param = { "/dev/ttyS1", TMT_B96, TMT_FRTSCTS };
GraphOpts graph_options = { FALSE, FALSE, TMT_GDFULL };

static GdkPixmap *pixmap = NULL;
static GtkWidget *dwgarea = NULL;
static TdsWave   *twave;
char *filename = NULL;    /* file name of the current wave */
int isntsaved = FALSE;

int
main (int argc, char *argv[])
{
	GtkWidget *mainw;
	GtkWidget *display;

    int c;
    int digit_optind = 0;
    int acq = FALSE;
	char outfile[255];
	Location src;

	strcpy (outfile, "TDSWaveData");

    while (1) {
		int this_option_optind = optind ? optind : 1;
		int option_index = 0;
		
		static struct option long_options[] = {
	    	{"help", 0, 0, 'h'},
	    	{"print", 1, 0, 'p'},
	    	{"acquire", 1, 0, 'a'},
	    	{"comm", 1, 0, 'c'},
	    	{"out", 1, 0, 'o'},
	    	{0, 0, 0, 0}
		};

		c = getopt_long(argc, argv, "hp:a:o:c:",
			long_options, &option_index);
		if (c == -1)
	    	break;

		switch (c) {
		case 0:
	    	printf("option %s", long_options[option_index].name);
	    	if (optarg)
				printf(" with arg %s", optarg);
	    	printf("\n");
	    	break;

		case 'p':
	    	printf("print waveform `%s'\n", optarg);
	    	cl_print(optarg);
	    	return 0;
	    	break;

		case 'a':
	    	printf("downloading from oscilloscope...\n");
	    	if (!strcmp(optarg, "ch1") || !strcmp(optarg, "1"))
	    		src = TMT_CH1;
	    	else if (!strcmp(optarg, "ch2") || !strcmp(optarg, "2"))
	    		src = TMT_CH2;
	    	else if (!strcmp(optarg, "math") || !strcmp(optarg, "m"))
	    		src = TMT_MATH;
	    	else if (!strcmp(optarg, "refa") || !strcmp(optarg, "a"))
	    		src = TMT_REFA;
	    	else if (!strcmp(optarg, "refb") || !strcmp(optarg, "b"))
	    		src = TMT_REFB;
	    	else {
	    		printf("bad argument for -a option\n");
	    		cl_help();
	    		return -1;
	    	}
	    	acq = TRUE;
	    	break;

		case 'c':
	    	if (!strcmp(optarg, "0"))
	    		strcpy(comm_param.device, "/dev/ttyS0");
	    	else if (!strcmp(optarg, "1"))
	    		strcpy(comm_param.device, "/dev/ttyS1");
	    	else if (!strcmp(optarg, "2"))
	    		strcpy(comm_param.device, "/dev/ttyS2");
	    	else if (!strcmp(optarg, "3"))
	    		strcpy(comm_param.device, "/dev/ttyS3");
	    	else {
	    		printf("bad argument for -c option\n");
	    		cl_help();
	    		return -1;
	    	}
	    	break;

		case 'o':
	    	strcpy(outfile, optarg);
	    	break;

		case 'h':
			cl_help();
			return 0;
			break;
			
		default:
	    	printf("?? getopt returned character code 0%o ??\n", c);
		}
	}

    if (optind < argc) {
		printf("non-option ARGV-elements: ");
		while (optind < argc)
	    	printf("%s ", argv[optind++]);
		printf("\n");
    }

	if (acq) {
		cl_acquire(src, outfile);
		return 0;
	}

	gtk_set_locale ();
	gtk_init (&argc, &argv);

	display = create_display (&dwgarea);
	gtk_widget_show (display);

	mainw = create_main_window ();
	gtk_widget_show (mainw);

	twave = (TdsWave *) malloc (sizeof (TdsWave));

	gtk_main ();
	return 0;
}


gboolean
closeprogram (GtkWidget *widget,
			   GdkEvent *event,
			   gpointer udata)
{
	if (isntsaved) {
		show_message_box ("File non saved!\n");
		return TRUE;
	}
	else
		return FALSE;
}


void
endprogram (GtkObject *object,
			 gpointer udata)
{
	gtk_main_quit ();
}


gboolean
closedisplay (GtkWidget *widget,
			   GdkEvent *event,
			   gpointer udata)
{
	show_message_box ("Close from main menu!\n");
	return FALSE;
}


void
on_open_activate (GtkMenuItem *menuitem,
				  gpointer udata)
{
	GtkWidget *filesel;

	filesel = gtk_file_selection_new ("Open File");

	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
						"clicked",
						GTK_SIGNAL_FUNC (on_openokb_clicked),
						filesel);

	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
										  (filesel)->cancel_button),
							   "clicked",
							   GTK_SIGNAL_FUNC (gtk_widget_destroy),
							   GTK_OBJECT (filesel));

	gtk_widget_show (filesel);
}


void
on_save_activate (GtkMenuItem *menuitem,
				  gpointer udata)
{
	if (filename != NULL) {
		FILE *fp;
		if ((fp = fopen (filename, "w")) != NULL) {
			fwrite (twave, sizeof(*twave), 1, fp);
			fclose (fp);
			isntsaved = FALSE;
		} else
			perror("fopen");
	}
	else   /*  saveas */
	{
		GtkWidget *filesel;
		filesel = gtk_file_selection_new ("Save File");

		gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION
									(filesel)->ok_button),
							"clicked",
							GTK_SIGNAL_FUNC (on_saveasokb_clicked),
							filesel);

		gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
									(filesel)->cancel_button),
								   "clicked",
								   GTK_SIGNAL_FUNC (gtk_widget_destroy),
								   GTK_OBJECT (filesel));

		gtk_widget_show (filesel);
	}
}


void
on_saveas_activate (GtkMenuItem *menuitem,
					 gpointer udata)
{
	GtkWidget *filesel;

	filesel = gtk_file_selection_new ("Save File");

	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
						"clicked",
						GTK_SIGNAL_FUNC (on_saveasokb_clicked),
						filesel);

	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
										  (filesel)->cancel_button),
							   "clicked",
							   GTK_SIGNAL_FUNC (gtk_widget_destroy),
							   GTK_OBJECT (filesel));

	gtk_widget_show (filesel);
}


void
on_print_activate (GtkMenuItem *menuitem,
				  gpointer main)
{
	FILE *fp;
	int i;
	
	if (filename == NULL) {
		show_message_box ("Error: nothing to print!\n");
		return;
	}
	if ((fp = fopen ("source.tmp", "w")) != NULL) {
		for (i = 0; i<2500; i++)
			fprintf (fp, "%i\n", twave->WFMdata[i]);
		fclose (fp);
	} else {
		perror("fopen");
		show_message_box ("Error: can't write temp file\n");
		return;
	}
	if ((fp = fopen ("plot.tmp", "w")) != NULL) {
		fprintf (fp, "set format ''\n");
		fprintf (fp, "set xtics border mirror norotate 250\n");
		fprintf (fp, "set ytics border mirror norotate 25.6\n");
		fprintf (fp, "set grid\n");
		fprintf (fp, "set nokey\n");
		fprintf (fp, "set xlabel '");
		fprintf (fp, "%s", twave->ID);
		fprintf (fp, "'\n");
		fprintf (fp, "set term postscript\n");
		fprintf (fp, "set output 'out.ps'\n");
		fprintf (fp, "plot[0:2500][0:255] 'source.tmp' w l, 128 w l\n");
		fprintf (fp, "set output '|lpr out.ps'\n");
		fprintf (fp, "quit");
		fclose (fp);
	} else {
		perror("fopen");
		show_message_box ("Error: can't write temp file.\n");
	}
	if (system("gnuplot plot.tmp"))
		show_message_box ("Error: Gnuplot not found\n");
	else
		show_message_box ("Output sent to gnuplot | lpr\n");
	unlink ("source.tmp");
	unlink ("plot.tmp");
}


void
on_quit_activate (GtkMenuItem *menuitem,
				  gpointer main)
{
	if (isntsaved) {
		show_message_box ("File non saved!\n");
		return ;
	}
	gtk_widget_destroy (main);
}


void
on_sendw_activate (GtkMenuItem *menuitem,
				  gpointer udata)
{
	GtkWidget *sw;
	if (filename == NULL) {
		show_message_box ("Load a waveform, first!\n");
		return;
	}
	if (comm_param.bps > TMT_B24)
		show_message_box("Baud rate too high! Set to 2400 or less.\n");
		return;
	sw = create_send_wave ();
	gtk_widget_show (sw);
}


void
on_receivew_activate (GtkMenuItem *menuitem,
					 gpointer udata)
{
	GtkWidget *rw;
	rw = create_receive_wave ();
	gtk_widget_show (rw);
}


void
on_sendsett_activate (GtkMenuItem *menuitem,
						   gpointer udata)
{
	GtkWidget *filesel;

	filesel = gtk_file_selection_new ("Open Settings File");

	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
						"clicked",
						GTK_SIGNAL_FUNC (on_sendsetokb_clicked),
						filesel);

	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
										  (filesel)->cancel_button),
							   "clicked",
							   GTK_SIGNAL_FUNC (gtk_widget_destroy),
							   GTK_OBJECT (filesel));

	gtk_widget_show (filesel);

}


void
on_sendsetokb_clicked (GtkButton *button,
							  GtkWidget *fs)
{
	FILE *fp;
	char *setfname;
	char msg[1024];
	char buf[2048];

	setfname = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));

	if ((fp = fopen (setfname, "r")) != NULL) {
		fread (buf, 2048, 1, fp);
		fclose (fp);
	} else
		perror("fopen");

	if ( strncmp (":HEAD", buf, 5) ) {
		sprintf (msg,"ERROR!\n\n %s isn't valid file!\n", setfname);
		show_message_box (buf);
		return;
	}

	txsettings (buf);
	gtk_widget_destroy (fs);
}


void
on_receivesett_activate (GtkMenuItem *menuitem,
							  gpointer udata)
{
	GtkWidget *filesel;

	filesel = gtk_file_selection_new ("Save Settings File");

	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
						"clicked",
						GTK_SIGNAL_FUNC (on_savesetokb_clicked),
						filesel);

	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
										  (filesel)->cancel_button),
							   "clicked",
							   GTK_SIGNAL_FUNC (gtk_widget_destroy),
							   GTK_OBJECT (filesel));

	gtk_widget_show (filesel);

}


void
on_savesetokb_clicked (GtkButton *button,
							  GtkWidget *fs)
{
	FILE *fp;
	char *setfname;
	char buf[2048];

	rxsettings (buf);

	setfname = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));

	if ((fp = fopen (setfname, "w")) != NULL) {
		fwrite (buf, 2048, 1, fp);
		fclose (fp);
	} else
		perror("fopen");

	gtk_widget_destroy (fs);
}


void
on_graphics_activate (GtkMenuItem *menuitem,
					  gpointer udata)
{
	GtkWidget *gopt;
	gopt = create_graph_options ();
	gtk_widget_show (gopt);
}


void
on_commport_activate (GtkMenuItem *menuitem,
					   gpointer udata)
{
	GtkWidget *cp;
	cp = create_comm_setup ();
	gtk_widget_show (cp);
}


void
on_about_activate (GtkMenuItem *menuitem,
				   gpointer udata)
{
	char msg[1024];
	sprintf (msg,
			 "TDS 200 mini Tool\n\n"
			 "A simple tool for \n"
			 "Tektronix TDS200-series oscilloscope\n\n"
			 "Version %s\n\n"
			 "By Alvaro De Amicis\n\n"
			 "Email: a.deamicis@iol.it\n",
			 VERSION);
	show_message_box (msg);
}


gboolean
close_display (GtkWidget *widget,
			   GdkEvent *event,
			   gpointer udata)
{

	return FALSE;
}


void
on_openokb_clicked (GtkButton *button,
						   GtkWidget *fs)
{
	FILE *fp;
	char msg[1024];

	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));

	if ((fp = fopen (filename, "r")) != NULL) {
		fread (twave, sizeof(*twave), 1, fp);
		fclose (fp);
	} else
		perror("fopen");
	if ( strncmp ("\"Ch1,", twave->WFMPreWFid, 5) &&
			strncmp ("\"Ch2,", twave->WFMPreWFid, 5) &&
			strncmp ("\"RefA,", twave->WFMPreWFid, 6) &&
			strncmp ("\"RefB,", twave->WFMPreWFid, 6) &&
			strncmp ("\"Math,", twave->WFMPreWFid, 6)
			) {
		sprintf (msg, "ERROR!\n\n%s isn't a valid file!\n", filename);
		show_message_box (msg);
		filename = NULL;
		return;
	}
	clear_drawing_area (dwgarea, pixmap);
	draw_grid (dwgarea, pixmap);
	draw_waveform (dwgarea, pixmap, twave);
	gtk_widget_destroy (fs);
}


void
on_saveasokb_clicked (GtkButton *button,
							 GtkWidget *fs)
{
	FILE *fp;
	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
	if ((fp = fopen (filename, "w")) != NULL) {
		fwrite (twave, sizeof(*twave), 1, fp);
		fclose (fp);
	} else
		perror("fopen");
	gtk_widget_destroy (fs);
	isntsaved = FALSE;
}


void
on_sfcb_clicked (GtkButton *button,
							 gpointer udata)
{
/*    unused function
 *   ( I used only the gtk_widget_destroy() )
 */
}


void
on_rwch1rb_toggled (GtkToggleButton *togglebutton,
					DialogDataL *dd)
{
	dd->location = TMT_CH1;
}


void
on_rwch2rb_toggled (GtkToggleButton *togglebutton,
					DialogDataL *dd)
{
	dd->location = TMT_CH2;
}


void
on_rwrefarb_toggled (GtkToggleButton *togglebutton,
					DialogDataL *dd)
{
	dd->location = TMT_REFA;
}


void
on_rwrefbrb_toggled (GtkToggleButton *togglebutton,
					DialogDataL *dd)
{
	dd->location = TMT_REFB;
}


void
on_rwmathrb_toggled (GtkToggleButton *togglebutton,
					DialogDataL *dd)
{
	dd->location = TMT_MATH;
}


void
on_rwreceiveb_clicked (GtkButton *button,
					   DialogDataL *dd)
{
	gtk_widget_destroy (dd->wnd);
	rxwaveform (twave, dd->location, TRUE);
	clear_drawing_area (dwgarea, pixmap);
	draw_grid (dwgarea, pixmap);
	draw_waveform (dwgarea, pixmap, twave);
	free (dd);
	isntsaved = TRUE;
	filename = "wave00";
}


void
on_rwcancb_clicked (GtkButton *button,
							 DialogDataL *dd)
{
	gtk_widget_destroy (dd->wnd);
	free (dd);
}


void
on_swrefarb_toggled (GtkToggleButton *togglebutton,
								DialogDataL *dd)
{
	dd->location = TMT_REFA;
}


void
on_swrefbrb_toggled (GtkToggleButton *togglebutton,
								DialogDataL *dd)
{
	dd->location = TMT_REFB;
}


void
on_swsendb_clicked (GtkButton *button,
						   DialogDataL *dd)
{
	gtk_widget_destroy (dd->wnd);
	if (twave!=NULL)
		txwaveform (twave, dd->location);
	free (dd);
}


void
on_swcancb_clicked (GtkButton *button,
							 DialogDataL *dd)
{
	gtk_widget_destroy (dd->wnd);
	free (dd);
}


void
on_cp96rb_toggled (GtkToggleButton *togglebutton,
								DialogData *dd)
{
	CommParam *p;
	p = dd->data;
	p->bps = TMT_B96;

}


void
on_cp192rb_toggled (GtkToggleButton *togglebutton,
								 DialogData *dd)
{
	CommParam *p;
	p = dd->data;
	p->bps = TMT_B192;

}


void
on_cp24rb_toggled (GtkToggleButton *togglebutton,
								DialogData *dd)
{
	CommParam *p;
	p = dd->data;
	p->bps = TMT_B24;
}


void
on_cp48rb_toggled (GtkToggleButton *togglebutton,
								DialogData *dd)
{
	CommParam *p;
	p = dd->data;
	p->bps = TMT_B48;
}


void
on_cp6rb_toggled (GtkToggleButton *togglebutton,
							   DialogData *dd)
{
	CommParam *p;
	p = dd->data;
	p->bps = TMT_B6;
}


void
on_cp12rb_toggled (GtkToggleButton *togglebutton,
								DialogData *dd)
{
	CommParam *p;
	p = dd->data;
	p->bps = TMT_B12;
}


void
on_cpctsrtsrb_toggled (GtkToggleButton *togglebutton,
								  DialogData *dd)
{
	CommParam *p;
	p = dd->data;
	p->flow = TMT_FRTSCTS;
}


void
on_cpxonxoffrb_toggled (GtkToggleButton *togglebutton,
								   DialogData *dd)
{
	CommParam *p;
	p = dd->data;
	p->flow = TMT_FXONXOFF;
}


void
on_cpnoflowrb_toggled (GtkToggleButton *togglebutton,
									DialogData *dd)
{
	CommParam *p;
	p = dd->data;
	p->flow = TMT_FNONE;
}


void
on_cpentry_activate (GtkEditable *entry,
					  DialogData  *dd)
{
	CommParam *p;
	char *text;
	p = dd->data;
	text = gtk_entry_get_text (GTK_ENTRY (entry));
	strncpy (p->device, text, 10);
	p->device[10] = '\0';
}


void
on_cpokb_clicked (GtkButton *button,
						 DialogData *dd)
{
	CommParam *p;
	p = dd->data;
	comm_param = *p;
	/*comm_param = (CommParam) dd->data;  ????????? */
	gtk_widget_destroy (dd->wnd);
}


void
on_cpcancb_clicked (GtkButton *button,
					gpointer udata)
{
	/* unused */
}


void
on_go_bg_chkbutton_toggled (GtkToggleButton *togglebutton,
							DialogData *dd)
{
	GraphOpts *go;
	go = dd->data;
	go->darkbg = (go->darkbg ? FALSE : TRUE) ;

}


void
on_go_mult_chkbutton_toggled (GtkToggleButton *togglebutton,
							  DialogData *dd)
{
	GraphOpts *go;
	go = dd->data;
	go->multiwf = (go->multiwf ? FALSE : TRUE) ;
}


void
on_go_grid_none_rbutton_toggled (GtkToggleButton *togglebutton,
								 DialogData *dd)
{
	GraphOpts *go;
	go = dd->data;
	go->grid = TMT_GDNONE;
}


void
on_go_grid_half_rbutton_toggled (GtkToggleButton *togglebutton,
								 DialogData *dd)
{
	GraphOpts *go;
	go = dd->data;
	go->grid = TMT_GDPOINTS ;
}


void
on_go_grid_full_rbutton_toggled (GtkToggleButton * togglebutton,
								 DialogData *dd)
{
	GraphOpts *go;
	go = dd->data;
	go->grid = TMT_GDFULL;
}


void
on_go_ok_button_clicked (GtkButton *button,
						 DialogData *dd)
{
	GraphOpts *go;
	go = dd->data;
	graph_options = *go;
	set_drawing_area (dwgarea, pixmap);
	gtk_widget_destroy (dd->wnd);
}


void
on_go_cancel_button_clicked (GtkButton *button,
							 gpointer udata)
{
	/* unused */
}


gboolean
on_dwgarea_configure_event (GtkWidget *widget,
							GdkEventConfigure *event)
{
	if (pixmap)
		gdk_pixmap_unref (pixmap);

	pixmap = gdk_pixmap_new (widget->window,
							 widget->allocation.width,
							 widget->allocation.height,
							 -1);
	gdk_draw_rectangle (pixmap,
						widget->style->white_gc,
						TRUE, 0, 0,
						widget->allocation.width,
						widget->allocation.height);

	draw_grid (widget, pixmap);
	return TRUE;
}


gboolean
on_dwgarea_expose_event (GtkWidget *widget,
						 GdkEventExpose *event)
{
	gdk_draw_pixmap (widget->window,
					 widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
					 pixmap,
					 event->area.x, event->area.y,
					 event->area.x, event->area.y,
					 event->area.width, event->area.height);
	return FALSE;
}
